@@ -7,12 +7,15 @@ import os
 import posixpath
 import re
 import sys
+import uuid
 import warnings
 from functools import singledispatch
 from typing import Iterator
 
 import h5py
 import numpy as np
+import prettytable
+from termcolor import colored
 
 from ._version import version  # noqa: F401
 from ._version import version_tuple  # noqa: F401
@@ -106,33 +109,73 @@ def join(*args, root: bool = False) -> str:
     return posixpath.join(*lst)
 
 
-def getdatapaths(file, root: str = "/") -> list[str]:
+def getdatapaths(
+    file: h5py.File,
+    root: str = "/",
+    max_depth: int = None,
+    fold: str | list[str] = None,
+    fold_symbol: str = "/...",
+) -> list[str]:
     """
     Get paths to all:
 
     -   Datasets, and
     -   Groups that contain attributes.
 
+    .. warning::
+
+        :py:func:`getgroups` visits all groups in the file,
+        regardless if they are folded (by ``fold`` or ``max_depth``).
+        Depending on the file, this can be quite costly.
+        If runtime is an issue consider searching for datasets only using :py:func:`getdatasets`
+        if your use-case allows it.
+
     :param file: A HDF5-archive.
-    :param root: Start at a certain point along the path-tree.
-    :return: List of paths.
+    :param root: Start a certain point along the path-tree.
+    :param max_depth: Set a maximum depth beyond which groups are folded.
+    :param fold: Specify groups that are folded.
+    :param fold_symbol: Use symbol to indicate that a group is folded.
+    :return: List of paths (always absolute, so includes the ``root`` if used).
     """
-    return list(getdatasets(file, root=root)) + list(getgroups(file, root=root, has_attrs=True))
+    kwargs = dict(root=root, max_depth=max_depth, fold=fold, fold_symbol=fold_symbol)
+    return list(getdatasets(file, **kwargs)) + list(getgroups(file, has_attrs=True, **kwargs))
 
 
 def getgroups(
-    file: h5py.File, root: str = "/", has_attrs: bool = False, max_depth: int = None
+    file: h5py.File,
+    root: str = "/",
+    has_attrs: bool = False,
+    max_depth: int = None,
+    fold: str | list[str] = None,
+    fold_symbol: str = "/...",
 ) -> list[str]:
     """
     Paths of all groups in a HDF5-archive.
 
+    .. warning::
+
+        The function visits all groups in the file,
+        regardless if they are folded (by ``fold`` or ``max_depth``).
+        Depending on the file, this can be quite costly.
+
     :param file: A HDF5-archive.
     :param root: Start at a certain point along the path-tree.
     :param has_attrs: Return only groups that have attributes.
     :param int max_depth: Set a maximum depth beyond which groups are folded.
-    :return: List of paths.
+    :param fold: Specify groups that are folded.
+    :param fold_symbol: Use symbol to indicate that a group is folded.
+    :return: List of paths (always absolute, so includes the ``root`` if used).
     """
 
+    root = abspath(root)
+
+    if fold:
+        if type(fold) is str:
+            fold = [abspath(fold)]
+        else:
+            fold = [abspath(f) for f in fold]
+        fold = [f if f.startswith(root) else join(root, f) for f in fold]
+
     keys = []
     group = file[root]
     group.visit(lambda key: keys.append(key) if isinstance(group[key], h5py.Group) else None)
@@ -145,14 +188,27 @@ def getgroups(
         n = len(list(filter(None, root.split("/"))))
         for i, path in enumerate(keys):
             if len(path.split("/")) - 1 > n + max_depth:
-                keys[i] = posixpath.join("/", *path.split("/")[: n + max_depth + 1]) + "/..."
+                keys[i] = posixpath.join("/", *path.split("/")[: n + max_depth + 1]) + fold_symbol
+        keys = list(set(keys))
+
+    if fold is not None:
+        if type(fold) is str:
+            fold = [fold]
+        for i, path in enumerate(keys):
+            for f in fold:
+                if path.startswith(f):
+                    keys[i] = f + fold_symbol
         keys = list(set(keys))
 
     return keys
 
 
 def getdatasets(
-    file: h5py.File, root: str = "/", max_depth: int = None, fold: str | list[str] = None
+    file: h5py.File,
+    root: str = "/",
+    max_depth: int = None,
+    fold: str | list[str] = None,
+    fold_symbol: str = "/...",
 ) -> Iterator:
     r"""
     Iterator to transverse all datasets in a HDF5-archive.
@@ -165,7 +221,8 @@ def getdatasets(
     :param root: Start a certain point along the path-tree.
     :param max_depth: Set a maximum depth beyond which groups are folded.
     :param fold: Specify groups that are folded.
-    :return: Iterator to paths.
+    :param fold_symbol: Use symbol to indicate that a group is folded.
+    :return: Iterator to paths (always absolute, so includes the ``root`` if used).
 
     :example:
 
@@ -201,20 +258,29 @@ def getdatasets(
         the second because it was specifically requested to be folded.
     """
 
-    if max_depth and fold:
-        return _getpaths_fold_maxdepth(file, root, fold, max_depth)
-
-    if max_depth:
-        return _getpaths_maxdepth(file, root, max_depth)
+    root = abspath(root)
 
     if fold:
-        return _getpaths_fold(file, root, fold)
+        if type(fold) is str:
+            fold = [abspath(fold)]
+        else:
+            fold = [abspath(f) for f in fold]
+        fold = [f if f.startswith(root) else join(root, f) for f in fold]
+
+    if max_depth and fold:
+        return _getpaths_fold_maxdepth(file, root, fold, max_depth, fold_symbol)
+
+    if max_depth:
+        return _getpaths_maxdepth(file, root, max_depth, fold_symbol)
+
+    if fold:
+        return _getpaths_fold(file, root, fold, fold_symbol)
 
     return _getpaths(file, root)
 
 
 def _getpaths(file, root):
-    r"""
+    """
     Specialization for :py:func:`getpaths`.
     """
 
@@ -241,8 +307,8 @@ def _getpaths(file, root):
     yield from iterator(file[root], root)
 
 
-def _getpaths_maxdepth(file, root, max_depth):
-    r"""
+def _getpaths_maxdepth(file, root, max_depth, fold_symbol):
+    """
     Specialization for :py:func:`getpaths` such that:
 
     - Groups deeper than a certain maximum depth are folded.
@@ -262,7 +328,7 @@ def _getpaths_maxdepth(file, root, max_depth):
 
             elif len(path.split("/")) - 1 >= max_depth:
                 if len(list(iterator(item, path, max_depth + 1))) > 0:
-                    yield path + "/..."
+                    yield path + fold_symbol
                 else:
                     yield path
 
@@ -280,8 +346,8 @@ def _getpaths_maxdepth(file, root, max_depth):
     yield from iterator(file[root], root, max_depth)
 
 
-def _getpaths_fold(file, root, fold):
-    r"""
+def _getpaths_fold(file, root, fold, fold_symbol):
+    """
     Specialization for :py:func:`getpaths` such that:
 
     - Certain groups are folded.
@@ -300,7 +366,7 @@ def _getpaths_fold(file, root, fold):
                 yield path
 
             elif path in fold:
-                yield path + "/..."
+                yield path + fold_symbol
 
             elif isinstance(item, h5py.Group):
                 yield from iterator(item, path, fold)
@@ -316,7 +382,7 @@ def _getpaths_fold(file, root, fold):
     yield from iterator(file[root], root, fold)
 
 
-def _getpaths_fold_maxdepth(file, root, fold, max_depth):
+def _getpaths_fold_maxdepth(file, root, fold, max_depth, fold_symbol):
     r"""
     Specialization for :py:func:`getpaths` such that:
 
@@ -337,10 +403,10 @@ def _getpaths_fold_maxdepth(file, root, fold, max_depth):
                 yield path
 
             elif len(path.split("/")) - 1 >= max_depth:
-                yield path + "/..."
+                yield path + fold_symbol
 
             elif path in fold:
-                yield path + "/..."
+                yield path + fold_symbol
 
             elif isinstance(item, h5py.Group):
                 yield from iterator(item, path, fold, max_depth)
@@ -532,6 +598,7 @@ def copy(
     source_datasets: list[str],
     dest_datasets: list[str] = None,
     root: str = None,
+    source_root: str = None,
     recursive: bool = True,
     skip: bool = False,
     expand_soft: bool = True,
@@ -540,13 +607,15 @@ def copy(
     Copy groups/datasets from one HDF5-archive ``source`` to another HDF5-archive ``dest``.
     The datasets can be renamed by specifying a list of ``dest_datasets``
     (whose entries should correspond to the ``source_datasets``).
-    In addition, a ``root`` (path prefix) for the destination datasets name can be specified.
+    In addition, a ``root`` path prefix can be specified for the destination datasets.
+    Likewise, a ``source_root`` path prefix can be specified for the source datasets.
 
     :param source: The source HDF5-archive.
     :param dest: The destination HDF5-archive.
     :param source_datasets: List of dataset-paths in ``source``.
     :param dest_datasets: List of dataset-paths in ``dest``, defaults to ``source_datasets``.
     :param root: Path prefix for all ``dest_datasets``.
+    :param source_root: Path prefix for all ``source_datasets``.
     :param recursive: If the source is a group, copy all objects within that group recursively.
     :param skip: Skip datasets that are not present in source.
     :param expand_soft: Copy the underlying data of a link, or copy as link with the same path.
@@ -555,12 +624,17 @@ def copy(
     if len(source_datasets) == 0:
         return
 
+    if type(source_datasets) is str:
+        source_datasets = [source_datasets]
+    if type(dest_datasets) is str:
+        dest_datasets = [dest_datasets]
+
     source_datasets = np.array([abspath(path) for path in source_datasets])
 
     if not dest_datasets:
-        dest_datasets = [path for path in source_datasets]
-
-    dest_datasets = np.array(dest_datasets)
+        dest_datasets = source_datasets.copy()
+    else:
+        dest_datasets = np.array([abspath(path) for path in dest_datasets])
 
     if skip:
         keep = [i in source for i in source_datasets]
@@ -568,11 +642,20 @@ def copy(
         dest_datasets = dest_datasets[keep]
 
     if root:
-        dest_datasets = np.array([join(root, path, root=True) for path in dest_datasets])
+        root = abspath(root)
+        dest_datasets = np.array([join(root, path) for path in dest_datasets])
+
+    if source_root:
+        source_root = abspath(source_root)
+        source_datasets = np.array([join(source_root, path) for path in source_datasets])
+
+    for path in source_datasets:
+        if path not in source:
+            raise OSError(f'Dataset "{path}" does not exists in source.')
 
     for path in dest_datasets:
-        if exists(dest, path):
-            raise OSError(f'Dataset "{path}" already exists')
+        if path in dest:
+            raise OSError(f'Dataset "{path}" already exists in dest.')
 
     isgroup = np.array([isinstance(source[path], h5py.Group) for path in source_datasets])
 
@@ -685,18 +768,20 @@ def _equal_value(a, b):
     return np.all(np.array(list(a[...])) == np.array(list(b[...])))
 
 
-def _equal(a, b, attrs, matching_dtype):
+def _equal(a, b, attrs, matching_dtype, shallow):
 
     if attrs:
 
         for key in a.attrs:
             if key not in b.attrs:
                 return False
-            if not _equal_value(a.attrs[key], b.attrs[key]):
-                return False
             if matching_dtype:
                 if a.attrs[key].dtype != b.attrs[key].dtype:
                     return False
+            if shallow:
+                return True
+            if not _equal_value(a.attrs[key], b.attrs[key]):
+                return False
 
         for key in b.attrs:
             if key not in a.attrs:
@@ -712,6 +797,9 @@ def _equal(a, b, attrs, matching_dtype):
         if a.dtype != b.dtype:
             return False
 
+    if shallow:
+        return True
+
     return _equal_value(a, b)
 
 
@@ -723,6 +811,7 @@ def equal(
     root: str = None,
     attrs: bool = True,
     matching_dtype: bool = False,
+    shallow: bool = False,
 ):
     r"""
     Check that a dataset is equal in both files.
@@ -734,6 +823,7 @@ def equal(
     :param root: Path prefix for ``dest_dataset``.
     :param attrs: Compare attributes (the same way at datasets).
     :param matching_dtype: Check that not only the data but also the type matches.
+    :param shallow: Check only the presence of the dataset, not its value.
     """
 
     if not dest_dataset:
@@ -748,7 +838,7 @@ def equal(
     if dest_dataset not in dest:
         raise OSError(f'"{dest_dataset:s} not in {dest.filename:s}')
 
-    return _equal(source[source_dataset], dest[dest_dataset], attrs, matching_dtype)
+    return _equal(source[source_dataset], dest[dest_dataset], attrs, matching_dtype, shallow)
 
 
 def allequal(
@@ -759,6 +849,7 @@ def allequal(
     root: str = None,
     attrs: bool = True,
     matching_dtype: bool = False,
+    shallow: bool = False,
 ):
     r"""
     Check that all listed datasets are equal in both files.
@@ -770,6 +861,7 @@ def allequal(
     :param root: Path prefix for all ``dest_datasets``.
     :param attrs: Compare attributes (the same way at datasets).
     :param matching_dtype: Check that not only the data but also the type matches.
+    :param shallow: Check only the presence of the dataset, not its value.
     """
 
     if not dest_datasets:
@@ -784,6 +876,7 @@ def allequal(
             root=root,
             attrs=attrs,
             matching_dtype=matching_dtype,
+            shallow=shallow,
         ):
             return False
 
@@ -791,28 +884,56 @@ def allequal(
 
 
 def _compare_paths(
-    a: h5py.File, b: h5py.File, paths_a: list[str], paths_b: list[str], attrs: bool
+    a: h5py.File,
+    b: h5py.File,
+    paths_a: list[str],
+    paths_b: list[str],
+    attrs: bool,
+    max_depth: int,
+    fold: str[list],
 ) -> type[list[str], list[str]]:
     """
     Default paths for :py:func:`compare`.
     """
 
+    if fold:
+        if type(fold) is str:
+            fold = [fold]
+
+    symbol = "/..." + str(uuid.uuid4())
+    fold_a = []
+    fold_b = []
+
     if paths_b is None and paths_a is not None:
         paths_b = paths_a
 
     if paths_a is None:
         if attrs:
-            paths_a = getdatapaths(a)
+            paths_a = getdatapaths(a, max_depth=max_depth, fold=fold, fold_symbol=symbol)
         else:
-            paths_a = list(getdatasets(a))
+            paths_a = list(getdatasets(a, max_depth=max_depth, fold=fold, fold_symbol=symbol))
 
     if paths_b is None:
         if attrs:
-            paths_b = getdatapaths(b)
+            paths_b = getdatapaths(b, max_depth=max_depth, fold=fold, fold_symbol=symbol)
         else:
-            paths_b = list(getdatasets(b))
+            paths_b = list(getdatasets(b, max_depth=max_depth, fold=fold, fold_symbol=symbol))
 
-    return paths_a, paths_b
+    if fold:
+
+        fold = [join(f, symbol, root=True) for f in fold]
+
+        for path in paths_a:
+            if path in fold:
+                paths_a.remove(path)
+                fold_a.append(path.split(symbol)[0])
+
+        for path in paths_b:
+            if path in fold:
+                paths_b.remove(path)
+                fold_b.append(path.split(symbol)[0])
+
+    return paths_a, paths_b, fold_a, fold_b
 
 
 @singledispatch
@@ -823,7 +944,11 @@ def compare(
     paths_b: list[str] = None,
     attrs: bool = True,
     matching_dtype: bool = False,
-):
+    shallow: bool = False,
+    only_datasets: bool = False,
+    fold: str | list[str] = None,
+    max_depth: int = None,
+) -> dict[list]:
     """
     Compare two files.
     Return dictionary with differences::
@@ -835,12 +960,22 @@ def compare(
             "==" : ["/data/matching", ...]
         }
 
+    .. warning::
+
+        Folded groups are not compared in any way!
+        Use `list_folded` to include this in the output.
+
     :param a: HDF5-archive (as opened ``h5py.File`` or with the ``filepath``).
     :param b: HDF5-archive (as opened ``h5py.File`` or with the ``filepath``).
     :param paths_a: Paths from ``a`` to consider. Default: read from :py:func:`getdatapaths`.
     :param paths_b: Paths from ``b`` to consider. Default: read from :py:func:`getdatapaths`.
     :param attrs: Compare attributes (the same way at datasets).
     :param matching_dtype: Check that not only the data but also the type matches.
+    :param shallow: Check only the presence of datasets, not their values, size, or attributes.
+    :param only_datasets: Compare datasets only (not groups, regardless if they have attributes).
+    :param max_depth: Set a maximum depth beyond which groups are folded.
+    :param fold: Specify groups that are folded.
+    :param list_folded: Return folded groups under `"??"`
     :return: Dictionary with difference.
     """
     raise NotImplementedError("Overload not found.")
@@ -854,10 +989,17 @@ def _(
     paths_b: list[str] = None,
     attrs: bool = True,
     matching_dtype: bool = False,
-):
+    shallow: bool = False,
+    only_datasets: bool = False,
+    max_depth: int = None,
+    fold: str | list[str] = None,
+    list_folded: bool = False,
+) -> dict[list]:
 
     ret = {"<-": [], "->": [], "!=": [], "==": []}
-    paths_a, paths_b = _compare_paths(a, b, paths_a, paths_b, attrs)
+    paths_a, paths_b, fold_a, fold_b = _compare_paths(
+        a, b, paths_a, paths_b, False if only_datasets else attrs, max_depth, fold
+    )
 
     not_in_b = [str(i) for i in np.setdiff1d(paths_a, paths_b)]
     not_in_a = [str(i) for i in np.setdiff1d(paths_b, paths_a)]
@@ -870,11 +1012,14 @@ def _(
         ret["->"].append(path)
 
     for path in inboth:
-        if not equal(a, b, path, attrs=attrs, matching_dtype=matching_dtype):
+        if not equal(a, b, path, attrs=attrs, matching_dtype=matching_dtype, shallow=shallow):
             ret["!="].append(path)
         else:
             ret["=="].append(path)
 
+    if list_folded:
+        ret["??"] = [str(i) for i in np.unique(fold_a + fold_b)]
+
     return ret
 
 
@@ -886,10 +1031,27 @@ def _(
     paths_b: list[str] = None,
     attrs: bool = True,
     matching_dtype: bool = False,
-):
+    shallow: bool = False,
+    only_datasets: bool = False,
+    max_depth: int = None,
+    fold: str | list[str] = None,
+    list_folded: bool = False,
+) -> dict[list]:
 
     with h5py.File(a, "r") as a_file, h5py.File(b, "r") as b_file:
-        return compare(a_file, b_file, paths_a, paths_a, attrs, matching_dtype)
+        return compare(
+            a_file,
+            b_file,
+            paths_a,
+            paths_b,
+            attrs,
+            matching_dtype,
+            shallow,
+            only_datasets,
+            max_depth,
+            fold,
+            list_folded,
+        )
 
 
 def compare_rename(
@@ -900,7 +1062,13 @@ def compare_rename(
     paths_b: list[str] = None,
     attrs: bool = True,
     matching_dtype: bool = False,
-):
+    shallow: bool = False,
+    regex: bool = False,
+    only_datasets: bool = True,
+    max_depth: int = None,
+    fold: str | list[str] = None,
+    list_folded: bool = False,
+) -> dict[list]:
     """
     Compare two files.
     Return three dictionaries with differences::
@@ -928,6 +1096,11 @@ def compare_rename(
             "==" : ["/path/in/b/with/rename/path/matching", ...]
         }
 
+    .. warning::
+
+        Folded groups are not compared in any way!
+        Use `list_folded` to include this in the output.
+
     :param a: HDF5-archive (as opened ``h5py.File`` or with the ``filepath``).
     :param b: HDF5-archive (as opened ``h5py.File`` or with the ``filepath``).
     :param rename: List with with renamed pairs: ``[["/a/0", "/b/1"], ...]``.
@@ -935,32 +1108,74 @@ def compare_rename(
     :param paths_b: Paths from ``b`` to consider. Default: read from :py:func:`getdatapaths`.
     :param attrs: Compare attributes (the same way at datasets).
     :param matching_dtype: Check that not only the data but also the type matches.
+    :param shallow: Check only the presence of datasets, not their values, size, or attributes.
+    :param regex: Use regular expressions to match ``rename``.
+    :param only_datasets: Compare datasets only (not groups, regardless if they have attributes).
+    :param max_depth: Set a maximum depth beyond which groups are folded.
+    :param fold: Specify groups that are folded.
+    :param list_folded: Return folded groups under `"??"`
+    :return: Dictionary with difference.
     """
 
+    opts = dict(
+        attrs=attrs,
+        matching_dtype=matching_dtype,
+        shallow=shallow,
+        max_depth=max_depth,
+        fold=fold,
+    )
+
     if rename is None:
         empty = {"!=": [], "==": []}
-        return compare(a, b, paths_a, paths_b, attrs, matching_dtype), empty, empty
+        return compare(a, b, paths_a, paths_b, **opts), empty, empty
 
-    paths_a, paths_b = _compare_paths(a, b, paths_a, paths_b, attrs)
+    paths_a, paths_b, fold_a, fold_b = _compare_paths(
+        a, b, paths_a, paths_b, False if only_datasets else attrs, max_depth, fold
+    )
 
-    for path_a, path_b in rename:
-        if path_a not in paths_a or path_b not in paths_b:
-            raise OSError("Renamed paths must be present")
-        paths_a.remove(path_a)
-        paths_b.remove(path_b)
+    rename_a = []
+    rename_b = []
 
-    ret = compare(a, b, paths_a, paths_b, attrs, matching_dtype)
+    if not regex:
+        for path_a, path_b in rename:
+            if path_a not in paths_a or path_b not in paths_b:
+                raise OSError("Renamed paths must be present")
+            rename_a.append(path_a)
+            rename_b.append(path_b)
+    else:
+        for pattern_a, pattern_b in rename:
+            for path_a in paths_a:
+                if re.match(rf"({pattern_a})(.*)", path_a):
+                    path_b = re.sub(rf"({pattern_a})(.*)", rf"{pattern_b}\2", path_a)
+                    if path_b not in paths_b:
+                        continue
+                    rename_a.append(path_a)
+                    rename_b.append(path_b)
+
+    keep_a = ~np.in1d(paths_a, rename_a)
+    keep_b = ~np.in1d(paths_b, rename_b)
+
+    paths_a = np.array(paths_a)[keep_a]
+    paths_b = np.array(paths_b)[keep_b]
+
+    ret = compare(a, b, paths_a, paths_b, **opts)
     ret_a = {"!=": [], "==": []}
     ret_b = {"!=": [], "==": []}
 
-    for path_a, path_b in rename:
-        if not equal(a, b, path_a, path_b, attrs=attrs, matching_dtype=matching_dtype):
+    opts.pop("max_depth")
+    opts.pop("fold")
+
+    for path_a, path_b in zip(rename_a, rename_b):
+        if not equal(a, b, path_a, path_b, **opts):
             ret_a["!="].append(path_a)
             ret_b["!="].append(path_b)
         else:
             ret_a["=="].append(path_a)
             ret_b["=="].append(path_b)
 
+    if list_folded:
+        ret["??"] = [str(i) for i in np.unique(fold_a + fold_b)]
+
     return ret, ret_a, ret_b
 
 
@@ -1033,9 +1248,17 @@ def print_plain(source, paths: list[str], show_links: bool = False):
         print(path)
 
 
-def print_info(source, paths: list[str], link_type: bool = False):
+def info_table(source, paths: list[str], link_type: bool = False) -> prettytable.PrettyTable:
     r"""
-    Print the paths to all datasets (one per line), including type information.
+    Get a table with basic information per path:
+
+    - path
+    - size
+    - shape
+    - dtype
+    - attrs: Number of attributes
+    - link: Link type
+
     :param paths: List of paths.
     :param link_type: Include the link-type in the output.
     """
@@ -1046,7 +1269,8 @@ def print_info(source, paths: list[str], link_type: bool = False):
                 return True
         return False
 
-    out = {"path": [], "size": [], "shape": [], "dtype": [], "attrs": []}
+    header = ["path", "size", "shape", "dtype", "attrs"]
+    out = {key: [] for key in header}
 
     for path in paths:
         if path in source:
@@ -1069,31 +1293,14 @@ def print_info(source, paths: list[str], link_type: bool = False):
             out["attrs"] += ["-"]
 
     if link_type:
+        header.append("link")
         out["link"] = [_linktype2str(source, path) for path in paths]
 
-    width = {}
-    for key in out:
-        width[key] = max(len(i) for i in out[key])
-        width[key] = max(width[key], len(key))
+    table = prettytable.PrettyTable()
+    for key in header:
+        table.add_column(column=out[key], fieldname=key, align="l")
 
-    fmt = "{path:%ds} {size:%ds} {shape:%ds} {dtype:%ds}" % (
-        width["path"],
-        width["size"],
-        width["shape"],
-        width["dtype"],
-    )
-
-    if has_attributes(out["attrs"]):
-        fmt += " {attrs:%ds}" % width["attrs"]
-
-    if "link" in out:
-        fmt += " {link:%ds}" % width["link"]
-
-    print(fmt.format(**{key: key for key in out}))
-    print(fmt.format(**{key: "=" * width[key] for key in out}))
-
-    for i in range(len(out["path"])):
-        print(fmt.format(**{key: out[key][i] for key in out}))
+    return table
 
 
 def print_attribute(source, paths: list[str]):
@@ -1218,22 +1425,19 @@ def G5print(args: list[str]):
                 print("")
 
 
-def _G5print_catch():
-    try:
-        G5print(sys.argv[1:])
-    except Exception as e:
-        print(e)
-        return 1
-
-
 def _G5list_parser():
     """
     Return parser for :py:func:`G5list`.
     """
 
     desc = """List datasets (or groups of datasets) in a HDF5-file.
-    Note that if you have a really big file the ``--layer`` option can be much faster than
-    searching the entire file.
+
+    If you have a really big file:
+
+    *   The ``--layer`` option can be much faster than searching the entire file.
+
+    *   Search only ``--datasets`` when folding can be much faster.
+        The ``getgroups`` function does search the entire depth.
     """
     parser = argparse.ArgumentParser(description=desc)
     parser.add_argument("--min-attrs", type=int, help="Filter based on min. number of attributes")
@@ -1245,7 +1449,8 @@ def _G5list_parser():
     parser.add_argument("-L", "--layer", type=str, help="Print paths at a specific layer")
     parser.add_argument("-l", "--long", action="store_true", help="--info but with attributes")
     parser.add_argument("-r", "--root", default="/", help="Start somewhere in the path-tree")
-    parser.add_argument("-s", "--links", action="store_true", help="Show destination of soft links")
+    parser.add_argument("--links", action="store_true", help="Show destination of soft links")
+    parser.add_argument("-s", "--sort", type=str, default="path", help="Sort by some column")
     parser.add_argument("-v", "--version", action="version", version=version)
     parser.add_argument("source")
     return parser
@@ -1286,16 +1491,197 @@ def G5list(args: list[str]):
                 paths.remove(path)
 
         if args.info:
-            print_info(source, paths, link_type=args.link_type)
+            table = info_table(source, paths, link_type=args.link_type)
+            table.set_style(prettytable.SINGLE_BORDER)
+            print(table.get_string(sortby=args.sort))
         elif args.long:
             print_attribute(source, paths)
         else:
             print_plain(source, paths, show_links=args.links)
 
 
+def _G5compare_parser():
+    """
+    Return parser for :py:func:`G5compare`.
+    """
+
+    desc = """Compare two HDF5 files.
+    If the function does not output anything all datasets are present in both files,
+    and all the content of the datasets is equal.
+    Each output line corresponds to a mismatch between the files.
+
+    Deal with renamed paths::
+
+        G5compare a.h5 b.h5 -r "/a" "/b"  -r "/c" "/d"
+    """
+    parser = argparse.ArgumentParser(description=desc)
+    parser.add_argument(
+        "-t", "--dtype", action="store_true", help="Verify that the type of the datasets match."
+    )
+    parser.add_argument(
+        "--shallow",
+        action="store_true",
+        help="Only check for the presence of datasets and attributes. Do not check their values.",
+    )
+    parser.add_argument("-D", "--datasets", action="store_true", help="Print only datasets")
+    parser.add_argument("-d", "--max-depth", type=int, help="Maximum depth to display")
+    parser.add_argument("-f", "--fold", action="append", help="Fold paths")
+    parser.add_argument("-r", "--renamed", nargs=2, action="append", help="Renamed paths.")
+    parser.add_argument("-c", "--colors", default="dark", help="Color theme: dark/light/none")
+    parser.add_argument("--table", default="SINGLE_BORDER", help="Table theme")
+    parser.add_argument("-v", "--version", action="version", version=version)
+    parser.add_argument("a", help="Path to HDF5-file.")
+    parser.add_argument("b", help="Path to HDF5-file.")
+    return parser
+
+
+def _truncate_print_path(path: str, n: int) -> str:
+    """
+    Truncate a file path, such that the printed length is smaller than ``n``.
+    For example::
+
+        "/path/to/long/foo/bar" -> "/path/.../bar"
+
+    :param path: Path to truncate.
+    :param n: Maximum length of the path.
+    :return: Truncated path.
+    """
+
+    if len(path) <= n:
+        return path
+
+    path = os.path.abspath(path).split(os.sep)
+    if len(path[0]) == 0:
+        path[0] = os.sep
+
+    length = np.array([len(i) for i in path])
+    incl = np.zeros(len(path), dtype=bool)
+    incl[0] = True
+    incl[-1] = True
+
+    # try including from the back
+    for i in range(1, len(path) - 1)[::-1]:
+        incl[i] = True
+        if np.sum(length[incl]) + np.sum(incl) + 3 > n:
+            incl[i] = False
+            break
+
+    # try including from the front
+    for i in range(1, len(path) - 1):
+        incl[i] = True
+        if np.sum(length[incl]) + np.sum(incl) + 3 > n:
+            incl[i] = False
+            break
+
+    if np.sum(incl) != len(path):
+        i = np.argmin(incl)
+        j = i + np.argmax(incl[i:])
+        path = path[:i] + ["..."] + path[j:]
+
+    return os.path.join(*path)
+
+
+def G5compare(args: list[str]):
+    """
+    Command-line tool to print datasets from a file, see ``--help``.
+    :param args: Command-line arguments (should be all strings).
+    """
+
+    parser = _G5compare_parser()
+    args = parser.parse_args(args)
+
+    for filepath in [args.a, args.b]:
+        if not os.path.isfile(filepath):
+            raise OSError(f'"{filepath}" does not exist')
+
+    with h5py.File(args.a, "r") as a, h5py.File(args.b, "r") as b:
+        comp, r_a, r_b = compare_rename(
+            a,
+            b,
+            rename=args.renamed,
+            matching_dtype=args.dtype,
+            shallow=args.shallow,
+            regex=True,
+            only_datasets=args.datasets,
+            fold=args.fold,
+            max_depth=args.max_depth,
+            list_folded=True,
+        )
+
+    def print_path(path):
+        if len(os.path.relpath(path).split("..")) < 3:
+            return os.path.relpath(path)
+        return os.path.abspath(path)
+
+    def def_row(arg, colors):
+
+        if colors == "none":
+            if arg[1] == "!=":
+                return arg
+            if arg[1] == "->":
+                return [arg[0], arg[1], ""]
+            if arg[1] == "<-":
+                return ["", arg[1], arg[2]]
+            if arg[1] == "??":
+                return [arg[0], arg[1], arg[2]]
+            if arg[1] != "==":
+                raise ValueError(f"Unknown operator {arg[1]}")
+
+        if arg[1] == "!=":
+            opts = dict(color="cyan", attrs=["bold"])
+            return [colored(arg[0], **opts), arg[1], colored(arg[2], **opts)]
+        if arg[1] == "->":
+            return [colored(arg[0], "red", attrs=["bold", "concealed"]), arg[1], ""]
+        if arg[1] == "<-":
+            return ["", arg[1], colored(arg[2], "green", attrs=["bold"])]
+        if arg[1] == "??":
+            opts = dict(color="magenta", attrs=["bold"])
+            return [colored(arg[0], **opts), arg[1], colored(arg[2], **opts)]
+        if arg[1] != "==":
+            raise ValueError(f"Unknown operator {arg[1]}")
+
+    out = prettytable.PrettyTable()
+    if args.table == "PLAIN_COLUMNS":
+        out.set_style(prettytable.PLAIN_COLUMNS)
+    elif args.table == "SINGLE_BORDER":
+        out.set_style(prettytable.SINGLE_BORDER)
+    out.align = "l"
+
+    for key in comp:
+        if key != "==":
+            for item in comp[key]:
+                out.add_row(def_row([item, key, item], args.colors))
+
+    for path_a, path_b in zip(r_a["!="], r_b["!="]):
+        out.add_row(def_row([path_a, "!=", path_b], args.colors))
+
+    file_a = print_path(args.a)
+    file_b = print_path(args.b)
+    n = max(len(row[0]) for row in out.rows)
+    out.field_names = [_truncate_print_path(file_a, n), "", _truncate_print_path(file_b, n)]
+
+    print(out.get_string())
+
+
+def _G5print_catch():
+    try:
+        G5print(sys.argv[1:])
+    except Exception as e:
+        print(e)
+        return 1
+
+
 def _G5list_catch():
     try:
         G5list(sys.argv[1:])
     except Exception as e:
         print(e)
         return 1
+
+
+def _G5compare_catch():
+    try:
+        G5compare(sys.argv[1:])
+    except Exception as e:
+        print(e)
+        return 1
